
#include "Header.h"

geBitmap *BitmapUtil_CreateFromFile(char *szName)
{
	geVFile  *file;
	geBitmap *bitmap;

	file = geVFile_OpenNewSystem(NULL,GE_VFILE_TYPE_DOS,szName,NULL,GE_VFILE_OPEN_READONLY);
	bitmap = geBitmap_CreateFromFile(file);

	geVFile_Close(file);

	return bitmap;
}

geBitmap *BitmapUtil_CreateFromFileAndAlpha(char *szName,char *szNameA)
{
	geBitmap *bitmap,*bitmapA;

	bitmap  = BitmapUtil_CreateFromFile(szName);
	bitmapA = BitmapUtil_CreateFromFile(szNameA);
	
	geBitmap_SetAlpha(bitmap,bitmapA);
	
	geBitmap_Destroy(&bitmapA);

	geBitmap_SetPreferredFormat(bitmap,GE_PIXELFORMAT_16BIT_4444_ARGB);

	return bitmap;
}

Corona::Corona()
{
	geWorld_GetEntitySet(CD->getm_World()->getWorld(),"Corona");
	bitmap = BitmapUtil_CreateFromFileAndAlpha("bmp\\corona.bmp","bmp\\corona_a.bmp");
	geWorld_AddBitmap(CD->getm_World()->getWorld(),bitmap);
}

Corona::~Corona()
{
	if(bitmap)
	{
		geWorld_RemoveBitmap(CD->getm_World()->getWorld(),bitmap);
		geBitmap_Destroy(&bitmap);
		bitmap = NULL;
	}
}

void Corona::Render()
{
	geEntity_EntitySet *entityset;
	geEntity           *entity;
	GE_Collision        collision;

	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"Corona");

	entity = geEntity_EntitySetGetNextEntity(entityset,NULL);

	while(entity)
	{
		CoronaDef  *Def;
		geVec3d		translation;
		geFloat		distanceToCorona;
		geVec3d		delta;
		geFloat		radius;
		geBoolean	visible;
		geBoolean	fading;
		int32		leaf;

		Def = (CoronaDef *)geEntity_GetUserData(entity);

		geWorld_GetLeaf(CD->getm_World()->getWorld(),&Def->origin,&leaf);

		if(geWorld_MightSeeLeaf(CD->getm_World()->getWorld(),leaf))
		{
			if(Def->Model)
			{
				geXForm3d xform;

				geWorld_GetModelXForm(CD->getm_World()->getWorld(),Def->Model,&xform);

				translation = Def->origin;

				if(Def->AllowRotation)
				{
					geVec3d	Center;

					geWorld_GetModelRotationalCenter(CD->getm_World()->getWorld(),Def->Model,&Center);
					geVec3d_Subtract(&translation,&Center,&translation);
					geXForm3d_Transform(&xform,&translation,&translation);
					geVec3d_Add(&translation,&Center,&translation);
				}
				else
					geVec3d_Add(&translation,&xform.Translation,&translation);
			}
			else
				translation = Def->origin;

			geVec3d_Subtract(&translation,&CD->getm_Camera()->getXForm().Translation,&delta);
			distanceToCorona = geVec3d_Length(&delta);

			if(!geWorld_Collision(CD->getm_World()->getWorld(),NULL,NULL,&translation,&CD->getm_Camera()->getXForm().Translation,GE_CONTENTS_CANNOT_OCCUPY,GE_COLLIDE_MODELS,0xffffffff,NULL,NULL,&collision) && (distanceToCorona < Def->MaxVisibleDistance))
			{
				visible = GE_TRUE;
				Def->LastVisibleTime = Def->LastTime;
			}
			else
				visible = GE_FALSE;

			fading = (Def->FadeOut && (Def->LastTime <= Def->LastVisibleTime + Def->FadeTime)) ? GE_TRUE : GE_FALSE;

			if(visible || fading)
			{
				GE_LVertex	Vert;
			
				Vert.X = translation.X;
				Vert.Y = translation.Y;
				Vert.Z = translation.Z;
				Vert.r = Def->Color.r;
				Vert.g = Def->Color.g;
				Vert.b = Def->Color.b;
				Vert.a = 255.0f;
				Vert.u = Vert.v = 0.0f;

				if(visible)
				{
					if(distanceToCorona >= Def->MaxRadiusDistance)
						radius = (geFloat)Def->MaxRadius;
					else 
					if(distanceToCorona <= Def->MinRadiusDistance)
						radius = (geFloat)Def->MinRadius;
					else
					{
						geFloat	slope;

						slope = (geFloat)(Def->MaxRadius - Def->MinRadius) / (geFloat)(Def->MaxRadiusDistance - Def->MinRadiusDistance);
						radius = (geFloat)Def->MinRadius + slope * (geFloat)(distanceToCorona - Def->MinRadiusDistance);
					}
				
					Def->LastVisibleRadius = radius;
				}
				else
				{
					assert(fading == GE_TRUE);
					assert(Def->LastTime <= Def->LastVisibleTime + Def->FadeTime);
					radius = (1.0f - (Def->LastTime - Def->LastVisibleTime) / Def->FadeTime) * Def->LastVisibleRadius;
				}
				
				geWorld_AddPolyOnce(CD->getm_World()->getWorld(),&Vert,1,bitmap,GE_TEXTURED_POINT,GE_RENDER_DO_NOT_OCCLUDE_OTHERS | GE_RENDER_DO_NOT_OCCLUDE_SELF,radius * 1);
			}
		
		Def->LastTime += 0.1f;
		}

		entity = geEntity_EntitySetGetNextEntity(entityset,entity);
	}
}

Light::Light()
{
}

Light::~Light()
{
	if(light)
		geWorld_RemoveLight(CD->getm_World()->getWorld(),light);
}

void Light::CreateLight(float X,float Y,float Z)
{
	light = geWorld_AddLight(CD->getm_World()->getWorld());

	translation.X = X;
	translation.Y = Y;
	translation.Z = Z;

	color.r = 255.0f;
	color.g = 255.0f;
	color.b = 255.0f;
	color.a = 255.0f;

	geWorld_SetLightAttributes(CD->getm_World()->getWorld(),light,&translation,&color,100.0f,GE_TRUE);
}

DynamicLight::DynamicLight()
{
	geEntity_EntitySet *entityset;
	geEntity           *entity;

	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"DynamicLight");

	if(entityset == NULL)
		return;

	entity = geEntity_EntitySetGetNextEntity(entityset,NULL);

	while(entity)
	{
#define MAX_NAME 200
		DynamicLightDef *Def;
		char             entityName[MAX_NAME];

		geEntity_GetName(entity,entityName,MAX_NAME - 1);
		
		entityName[MAX_NAME - 1] = 0;
		Def                      = (DynamicLightDef*)geEntity_GetUserData(entity);
		Def->NumFunctionValues   = strlen(Def->RadiusFunction);

		if(Def->NumFunctionValues == 0)
			return;

		Def->IntervalWidth = Def->RadiusSpeed / (geFloat)Def->NumFunctionValues;
		Def->DynLight = geWorld_AddLight(CD->getm_World()->getWorld());
		
		if(!Def->DynLight)
			return;
		
		entity = geEntity_EntitySetGetNextEntity(entityset,entity);	
	}
}

DynamicLight::~DynamicLight()
{
}

void DynamicLight::Render()
{
	geEntity_EntitySet *entityset;
	geEntity           *entity;
 
	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"DynamicLight");
	
	if(entityset == NULL)
		return;
		
	entity = geEntity_EntitySetGetNextEntity(entityset,NULL);
	
	while(entity)
	{
		DynamicLightDef *Def;
		geFloat	 	 	 Radius;
		geFloat	 		 Percentage;
		int				 Index;
		geVec3d			 Translation;

		Def = (DynamicLightDef *)geEntity_GetUserData(entity);
		
		if(Def->Model)
		{
			geXForm3d XForm;

			geWorld_GetModelXForm(CD->getm_World()->getWorld(),Def->Model,&XForm);

			Translation = Def->origin;
			
			if(Def->AllowRotation)
			{
				geVec3d	Center;

				geWorld_GetModelRotationalCenter(CD->getm_World()->getWorld(),Def->Model,&Center);
				geVec3d_Subtract(&Translation,&Center,&Translation);
				geXForm3d_Transform(&XForm,&Translation,&Translation);
				geVec3d_Add(&Translation,&Center,&Translation);
			}
			else
				geVec3d_Add(&Translation,&XForm.Translation,&Translation);
		}
		else
		{
			Translation = Def->origin;
		}

		Percentage = Def->LastTime / Def->RadiusSpeed;
		Index      = (int)(Percentage * Def->NumFunctionValues);

		if(Def->InterpolateValues && Index < Def->NumFunctionValues - 1)
		{
			geFloat	Remainder;
			geFloat	InterpolationPercentage;
			int		DeltaValue;
			geFloat	Value;

			Remainder               = (geFloat)fmod(Def->LastTime,Def->IntervalWidth);
			InterpolationPercentage = Remainder / Def->IntervalWidth;
			DeltaValue              = Def->RadiusFunction[Index + 1] - Def->RadiusFunction[Index];
			Value                   = Def->RadiusFunction[Index] + DeltaValue * InterpolationPercentage;
			Percentage              = ((geFloat)(Value - 'a')) / ((geFloat)('z' - 'a'));
		}
		else
			Percentage              = ((geFloat)(Def->RadiusFunction[Index] - 'a')) / ((geFloat)('z' - 'a'));
		
		Radius             = Percentage * (Def->MaxRadius - Def->MinRadius) + Def->MinRadius;

		geWorld_SetLightAttributes(CD->getm_World()->getWorld(),Def->DynLight,&Translation,&Def->Color,Radius,GE_TRUE);

		Def->LastTime = (geFloat)fmod(Def->LastTime + 0.05f,Def->RadiusSpeed);
		entity           = geEntity_EntitySetGetNextEntity(entityset,entity);
	}
}