
#pragma warning(disable : 4068)

#ifndef _EFFECT_H_
#define _EFFECT_H_

geBitmap *BitmapUtil_CreateFromFileAndAlpha(char *szName);
geBitmap *BitmapUtil_CreateFromFileAndAlpha(char *szName,char *szNameA);

#pragma GE_Type("corona.bmp")
typedef struct	CoronaDef
{
	#pragma	GE_Private
		geFloat			LastTime;
		geFloat			LastVisibleTime;
		geFloat			LastVisibleRadius;
	
	#pragma GE_Published
	    geVec3d			origin;
		int				FadeOut;
		geFloat			FadeTime;
	    int				MinRadius;
	    int				MaxRadius;
		int				MaxVisibleDistance;
		int				MaxRadiusDistance;
		int				MinRadiusDistance;
		int				AllowRotation;
		geWorld_Model *	Model;
	    GE_RGBA			Color;

	#pragma GE_Origin(origin)
	#pragma GE_DefaultValue(FadeOut,"1")
	#pragma GE_DefaultValue(FadeTime,"0.15")
	#pragma GE_DefaultValue(MinRadius,"1")
	#pragma GE_DefaultValue(MaxRadius,"20")
	#pragma GE_DefaultValue(MaxVisibleDistance,"2000")
	#pragma GE_DefaultValue(MaxRadiusDistance,"1000")
	#pragma GE_DefaultValue(MinRadiusDistance,"100")
	#pragma GE_DefaultValue(AllowRotation,"1")
	#pragma GE_DefaultValue(Color,"255.0 255.0 255.0")

	#pragma GE_Documentation(FadeOut,"Whether to fade out coronas when they pass out of visibility (0 or 1)")
	#pragma GE_Documentation(FadeTime,"How long the fade takes to drop to zero visibility (seconds)")
	#pragma GE_Documentation(MinRadius,"Minimum radius corona will ever drop to (texels)")
	#pragma GE_Documentation(MaxRadius,"Maximum radius corona will ever increase to (texels)")
	#pragma GE_Documentation(MaxVisibleDistance,"Maximum distance corona is visible at (texels)")
	#pragma GE_Documentation(MaxRadiusDistance,"Above this distance, corona is capped at MaxRadius (texels)")
	#pragma GE_Documentation(MinRadiusDistance,"Below this distance, corona is capped at MinRadius (texels)")
	#pragma GE_Documentation(AllowRotation,"Permit rotation about model, if one is present (0 or 1)")
	#pragma GE_Documentation(Model,"World model to slave motion of corona to")
}CoronaDef;

class Corona
{
public:
	Corona();
   ~Corona();

    void Render();
private:
	geBitmap *bitmap;
};

class Light
{
public:
	Light();
   ~Light();

    void CreateLight(float X,float Y,float Z);
private:
	geLight *light;
	geVec3d  translation;
	GE_RGBA  color;
};

#pragma GE_Type("dynlight.bmp")
typedef struct	DynamicLightDef
{
#pragma	GE_Private
	geFloat			LastTime;
	geFloat			IntervalWidth;
	int				NumFunctionValues;
	geLight        *DynLight;
	
#pragma GE_Published
    geVec3d			origin;
    int				MinRadius;
    int				MaxRadius;
	int				InterpolateValues;
	int				AllowRotation;
    char           *RadiusFunction;
    geFloat			RadiusSpeed;
	geWorld_Model  *Model;
    GE_RGBA			Color;

#pragma GE_Origin(origin)
#pragma GE_DefaultValue(MinRadius,"50")
#pragma GE_DefaultValue(MaxRadius,"300")
#pragma GE_DefaultValue(Color,"40.0 40.0 153.0")
#pragma GE_DefaultValue(RadiusFunction,"aza")
#pragma GE_DefaultValue(RadiusSpeed,"2.0")
#pragma GE_DefaultValue(InterpolateValues,"1")
#pragma GE_DefaultValue(AllowRotation,"1")

#pragma GE_Documentation(MinRadius,"Minimum radius of the light (texels)")
#pragma GE_Documentation(MaxRadius,"Maximum radius of the light (texels)")
#pragma GE_Documentation(InterpolateValues,"0 or 1.  Whether to interpolate between the RadiusFunction values")
#pragma GE_Documentation(AllowRotation,"0 or 1.  Whether or not to follow the model rotation if the light is attached to a model")
#pragma GE_Documentation(RadiusFunction,"a-z, repeated (E.g. aabdfzddsfdz)  Specify light values over time.  a = Minimum z = maximum")
#pragma GE_Documentation(RadiusSpeed,"How fast to run through RadiusFunction values (seconds)")
#pragma GE_Documentation(Model,"Model that the light is attached to.  Optional")
#pragma GE_Documentation(Color,"Color of the light")
}DynamicLightDef;

class DynamicLight
{
public:
	DynamicLight();
   ~DynamicLight();

    void Render();
private:
};

#endif