
#include "Header.h"

LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);

Engine::Engine()
{
	Width        = 640;
	Height       = 480;
	szName       = "Genesis3D Project";
	engine       = NULL;
	DrvName      = '(';
	DrvDirectory = ".";
	DrvSys       = NULL;
	Drv          = NULL;
	DrvMode      = NULL;
	Gamma        = 2.0f;
}

Engine::~Engine()
{
	if(SoundSys)
	{
		geSound_DestroySoundSystem(SoundSys);
		SoundSys = NULL;
	}

	if(engine)
	{
		geEngine_ShutdownDriver(engine);
		geEngine_Free(engine);
		engine = NULL;
	}
}

bool Engine::Create(HINSTANCE hInstance)
{
	FILE *file;
	char  tag[256];

	if((file = fopen("prefs.ini","r")) == NULL)
		return false;

	while(!feof(file))
	{
		fscanf(file,"%s",tag);

		if(!strcmp(tag,"RES_W"))
		{
			fscanf(file,"%d",&Width);
		}
		
		if(!strcmp(tag,"RES_H"))
		{
			fscanf(file,"%d",&Height);
		}
	}

	fclose(file);

	WNDCLASS wndclass;
	
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInstance;
	wndclass.lpfnWndProc   = WndProc;
	wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
	wndclass.hCursor       = LoadCursor(NULL,IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wndclass.lpszClassName = szName;;
	wndclass.lpszMenuName  = NULL;

	RegisterClass(&wndclass);

	hWnd = CreateWindowEx(0,szName,szName,WS_VISIBLE | WS_POPUP,0,0,640,480,NULL,NULL,hInstance,NULL);

	ShowWindow(hWnd,SW_SHOWNORMAL);
	UpdateWindow(hWnd);

	ShowCursor(FALSE);

	engine = geEngine_Create(hWnd,szName,DrvDirectory);

	if(!engine)
		return false;

	geEngine_EnableFrameRateCounter(engine,GE_FALSE);
	DrvSys = geEngine_GetDriverSystem(engine);

	FindDriver();

	geEngine_SetDriverAndMode(engine,Drv,DrvMode);
	geEngine_SetGamma(engine,Gamma);

	SoundSys = geSound_CreateSoundSystem(hWnd);

	if(!SoundSys)
		return false;
	
	geSound_SetMasterVolume(SoundSys,0.9f);

	return true;
}

bool Engine::FindDriver()
{
	long        getwidth,getheight;
	const char *name;

	Drv = geDriver_SystemGetNextDriver(DrvSys,NULL);

	while(true)
	{
		geDriver_GetName(Drv,&name);

		if(name[0] == DrvName)
			break;

		geDriver_SystemGetNextDriver(DrvSys,Drv);
	}

	DrvMode = geDriver_GetNextMode(Drv,NULL);

	while(true)
	{
		geDriver_ModeGetWidthHeight(DrvMode,&getwidth,&getheight);

		if(getwidth == Width && getheight == Height)
			break;

		DrvMode = geDriver_GetNextMode(Drv,DrvMode);
	}

	return true;
}

bool Engine::CreateEngine(HINSTANCE hInstance)
{
	Create(hInstance);

	return true;
}

int Engine::Begin()
{
	geEngine_BeginFrame(engine,CD->getm_Camera()->getCamera(),GE_TRUE);

	return 1;
}

int Engine::End()
{
	geEngine_EndFrame(engine);

	return 1;
}

int Engine::Render()
{
	geEngine_RenderWorld(engine,CD->getm_World()->getWorld(),CD->getm_Camera()->getCamera(),0.0f);

	return 1;
}

void Engine::Logo(char *szName)
{
	geBitmap     *bitmap;
	geBitmap_Info bitmapInfo;
	int           x,y;

	bitmap = geBitmap_CreateFromFileName(NULL,szName);

	if(geBitmap_GetInfo(bitmap,&bitmapInfo,NULL))
	{
		x = (Width - bitmapInfo.Width) / 2;
		y = (Height - bitmapInfo.Height) / 2;

		geEngine_AddBitmap(engine,bitmap);
	}

	Begin();

	geEngine_DrawBitmap(engine,bitmap,NULL,x,y);
	
	End();

	geEngine_RemoveBitmap(engine,bitmap);
	geBitmap_Destroy(&bitmap);
}

void Engine::FrameRate(bool enable)
{
	if(enable)
		geEngine_EnableFrameRateCounter(engine,GE_TRUE);
	else
		geEngine_EnableFrameRateCounter(engine,GE_FALSE);
}

void Engine::Print(int x,int y,char *szString)
{
	geEngine_Printf(engine,x,y,szString);
}
