
#include "Header.h"


ITEM_ENTITY_INFO *info1[MAX_NUMBER_OF_ITEMS];

void LoadEntity()
{
	LoadItem();
	LoadLight();
}

void LoadItem()
{
	geEntity_EntitySet *entityset;
	geEntity           *entity;
	geVFile            *file;
	geActor_Def        *def;
	geExtBox            box;
	int                 counter = -1;

	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"Item");

	if(entityset == NULL)
		return;

	for(int n = 0; n < MAX_NUMBER_OF_ITEMS; n++)
	{
		if(info1[n] == NULL)
			info1[n] = new ITEM_ENTITY_INFO;
	}

	for(entity = geEntity_EntitySetGetNextEntity(entityset,NULL); entity; entity = geEntity_EntitySetGetNextEntity(entityset,entity))
	{
		counter++;

		geXForm3d xform;
		geXForm3d_SetIdentity(&xform);

		ITEM *item = (ITEM *)geEntity_GetUserData(entity);

		file = geVFile_OpenNewSystem(NULL,GE_VFILE_TYPE_DOS,item->szName,NULL,GE_VFILE_OPEN_READONLY);

		if(file)
		{
			def = geActor_DefCreateFromFile(file);

			if(def)
			{
				item->actor = geActor_Create(def);

				geWorld_AddActor(CD->getm_World()->getWorld(),item->actor,GE_ACTOR_RENDER_NORMAL | GE_ACTOR_COLLIDE,0xffffffff);

				geActor_SetShadow(item->actor,GE_TRUE,10.0f,NULL,NULL);

				geActor_GetDynamicExtBox(item->actor,&box);
				geActor_SetExtBox(item->actor,&box,NULL);
				geActor_SetRenderHintExtBox(item->actor,&box,NULL);

				info1[counter]->type = item->Type;

				geActor_SetUserData(item->actor,(void *)info1[counter]);

				geActor_SetScale(item->actor,0.5f,0.5f,0.5f);

				geXForm3d_Translate(&xform,item->Origin.X,item->Origin.Y,item->Origin.Z);

				geVec3d pos = Gravity(item->Origin,box);

				item->Origin = pos;

				geXForm3d_SetIdentity(&xform);
				geXForm3d_RotateX(&xform,-1.5707f);
				geXForm3d_Translate(&xform,item->Origin.X,item->Origin.Y,item->Origin.Z);
				geActor_ClearPose(item->actor,&xform);
			}
			else
				return;
		}

		geVFile_Close(file);
	}
}			

void LoadLight()
{
	geEntity_EntitySet *entityset;
	geEntity           *entity;

	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"light");

	if(entityset == NULL)
		return;

	for(entity = geEntity_EntitySetGetNextEntity(entityset,NULL); entity; entity = geEntity_EntitySetGetNextEntity(entityset,entity))
	{
		LIGHT *l = (LIGHT *)geEntity_GetUserData(entity);

		l->light = geWorld_AddLight(CD->getm_World()->getWorld());

		geWorld_SetLightAttributes(CD->getm_World()->getWorld(),l->light,&l->Origin,&l->Color,l->Radius,GE_TRUE);
	}
}

void DeleteEntity()
{
	for(int n = 0; n < MAX_NUMBER_OF_ITEMS; n++)
	{
		if(info1[n])
			N_DELETE(info1[n]);
	}
}

geVec3d Gravity(geVec3d pos,geExtBox box)
{
	geVec3d      floor;
	geVec3d      direction;
	geXForm3d    xform;
	GE_Collision collision;
	
	geXForm3d_SetIdentity(&xform);
	
	geXForm3d_GetUp(&xform,&direction);
	geVec3d_AddScaled(&pos,&direction,-1.0f,&floor);

	bool collide = geWorld_Collision(CD->getm_World()->getWorld(),&box.Min,&box.Max,&pos,&floor,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision);

	if(collide)
		floor = collision.Impact;
		
	floor.Y += 1.0f;
	pos      = floor;

	return pos;
}

int EntityCollision(ITEM_ENTITY_INFO *info)
{
	switch(info->type)
	{
		case 0:
			return 0;
			break;
		case 1:
			return 1;
			break;
		case 2:
			return 2;
			break;
		case 3:
			return 3;
			break;
		case 4:
			return 4;
			break;
		default:
			break;
	}

	return -1;
}
