
#include "Header.h"


GameMenu::GameMenu()
{
	camera   = NULL;
	geVec3d_Set(&translation,0.0f,65.0f,40.0f);
	geVec3d_Set(&rotation,0.0f,0.0f,0.0f);

	m_Dummy = NULL;
	m_Light = NULL;
	
	for(int n = 0; n < MAX_NUMBER_OF_MENU_IMAGES; n++)
		m_Image[n] = NULL;
}

GameMenu::~GameMenu()
{
	for(int n = 0; n < MAX_NUMBER_OF_MENU_IMAGES; n++)
		N_DELETE(m_Image[n]);

	N_DELETE(m_Light);

	N_DELETE(m_Dummy);
	if(camera)
	{
		geCamera_Destroy(&camera);
		camera = NULL;
	}
}

void GameMenu::Load()
{
	rect.Left   = 0;
	rect.Top    = 0;
	rect.Right  = CD->getm_Engine()->getWidth() - 1;
	rect.Bottom = CD->getm_Engine()->getHeight() - 1;

	camera = geCamera_Create(2.0f,&rect);

	geXForm3d_SetIdentity(&xform);

	geXForm3d_RotateZ(&xform,rotation.Z);
	geXForm3d_RotateX(&xform,rotation.X);
	geXForm3d_RotateY(&xform,rotation.Y);
	geXForm3d_Translate(&xform,translation.X,translation.Y,translation.Z);

	m_Dummy = new Dummy();
	N_DETECT_NULL(m_Dummy,"Unable to allocate memory to m_Dummy");
	N_DETECT(m_Dummy->Load("actor\\spawn.act"),"failed to call m_Dummy->Load()");

	m_Light = new Light();
	N_DETECT_NULL(m_Light,"Unable to allocate memory to m_Light");
	m_Light->CreateLight(0.0f,0.0f,30.0f);

	for(int n = 0; n < MAX_NUMBER_OF_MENU_IMAGES; n++)
	{
		m_Image[n] = new Image();
		N_DETECT_NULL(m_Image[n],"Unable to allocate memory to m_Image");
	}

	N_DETECT(m_Image[MENU_IMAGE_NEWGAME]->Load("bmp\\m1.bmp",180,30),"failed to call m_Image[MENU_IMAGE_NEWGAME]->Load()");
	N_DETECT(m_Image[MENU_IMAGE_QUIT]->Load("bmp\\m2.bmp",180,30),"failed to call m_Image[MENU_IMAGE_QUIT]->Load()");
	N_DETECT(m_Image[MENU_IMAGE_OK]->Load("bmp\\m3.bmp",180,30),"failed to call m_Image[MENU_IMAGE_OK]->Load()");
	N_DETECT(m_Image[MENU_IMAGE_CANCEL]->Load("bmp\\m4.bmp",180,30),"failed to call m_Image[MENU_IMAGE_CANCEL]->Load()");

	m_Image[MENU_IMAGE_NEWGAME]->setXY(400,350);
	m_Image[MENU_IMAGE_QUIT]->setXY(400,400);
	m_Image[MENU_IMAGE_OK]->setXY(400,350);
	m_Image[MENU_IMAGE_CANCEL]->setXY(400,400);

	m_Image[MENU_IMAGE_NEWGAME]->setType(MENU_IMAGE_NEWGAME);
	m_Image[MENU_IMAGE_QUIT]->setType(MENU_IMAGE_QUIT);
	m_Image[MENU_IMAGE_OK]->setType(MENU_IMAGE_OK);
	m_Image[MENU_IMAGE_CANCEL]->setType(MENU_IMAGE_CANCEL);
}

void GameMenu::ProcessMenu()
{
	ShowCursor(true);

	geEngine_BeginFrame(CD->getm_Engine()->getEngine(),camera,GE_TRUE);
	
	for(int n = 0; n < MAX_NUMBER_OF_MENU_IMAGES; n++)
	{
		m_Image[n]->Draw();

		if(m_Image[n]->MouseOver() && CD->getm_Keyboard()->Get(VK_LBUTTON))
		{
			switch(m_Image[n]->getType())
			{
				case MENU_IMAGE_NEWGAME:
					CD->getm_Game()->setState(GAMESTATE_SELECT);
					break;
				case MENU_IMAGE_QUIT:
					CD->getm_Game()->setState(GAMESTATE_QUIT);
					break;
			}
		}
	}
		
	geEngine_EndFrame(CD->getm_Engine()->getEngine());
}

void GameMenu::ProcessSelect()
{
	geCamera_SetWorldSpaceXForm(camera,&xform);
	geEngine_BeginFrame(CD->getm_Engine()->getEngine(),camera,GE_TRUE);
	geEngine_RenderWorld(CD->getm_Engine()->getEngine(),CD->getm_World()->getWorld(),camera,0.0f);

	m_Dummy->Update();

	geEngine_EndFrame(CD->getm_Engine()->getEngine());
}