
#include "Header.h"


Player::Player()
{
	Actor         = NULL;
	Def           = NULL;
	Height        = 70.0f;
	Getup         = -1.5707f;
	motion        = NULL;
	Counter       = 0.0f;
	CurrentMotion = IDLE;
	Speed         = 4.0f;
	StepHeight    = 2.0f;

	OnGround      = false;
	Jumping       = false;
	Falling       = false;
	GravityOn     = true;
	Crouch        = false;
	Aim           = false;
	gravity       =-2.0f;
	Acceleration  = 0.0f;
	JumpHeight    = 0.0f;

	geVec3d_Set(&Translation,0.0f,0.0f,0.0f);
	geVec3d_Set(&Rotation,0.0f,0.0f,0.0f);

	Box.Min.X = Box.Min.Z = -16.0f;
	Box.Min.Y = 0.0f;
	Box.Max.X = Box.Max.Z =  16.0f;
	Box.Max.Y = Height;
}

Player::~Player()
{
	if(Def)
	{
		geActor_DefDestroy(&Def);
		Def = NULL;
	}

	if(Actor)
	{
		geActor_Destroy(&Actor);
		Actor = NULL;
	}
}

bool Player::Load(char *szName)
{
	File  = geVFile_OpenNewSystem(NULL,GE_VFILE_TYPE_DOS,szName,NULL,GE_VFILE_OPEN_READONLY);

	if(!File)
		return false;
	
	Def   = geActor_DefCreateFromFile(File);
	
	if(!Def)
		return false;

	Actor = geActor_Create(Def);
	
	if(!Actor)
		return false;

	geWorld_AddActor(CD->getm_World()->getWorld(),Actor,GE_ACTOR_RENDER_NORMAL | GE_ACTOR_COLLIDE,0xffffffff);

	geVec3d Normal = { 0.0f, 1.0f, 0.0f };
	geActor_SetLightingOptions(Actor,GE_TRUE,&Normal,255.0f,255.0f,255.0f,1.0f,1.0f,1.0f,GE_TRUE,3,NULL,GE_FALSE);

	geActor_SetExtBox(Actor,&Box,NULL);

	geXForm3d_SetIdentity(&XForm);

	geXForm3d_RotateZ(&XForm,Rotation.Z);
	geXForm3d_RotateX(&XForm,0.0f);
	geXForm3d_RotateY(&XForm,Rotation.Y + 3.14159f);
	geXForm3d_Translate(&XForm,Translation.X,Translation.Y,Translation.Z);

	geActor_ClearPose(Actor,&XForm);

	geVFile_Close(File);

	return true;
}

void Player::SetType(int type)
{
	Type = type;
}

void Player::Control()
{
	MouseLook();

	//-- Actor's Movement
	if(CD->getm_Keyboard()->GetDown(KEY_W))
	{
		Motion(RUN); Forward();
	}
	else
	if(CD->getm_Keyboard()->GetDown(KEY_S))
	{
		Motion(RUN); Backward();
	}
	else
	if(CD->getm_Keyboard()->GetDown(KEY_A))
	{
		Motion(STRAFELEFT); Left();
	}
	else
	if(CD->getm_Keyboard()->GetDown(KEY_D))
	{
		Motion(STRAFERIGHT); Right();
	}
		
	//-- Actor's Crouch
	if(CD->getm_Keyboard()->GetDown(KEY_C))
		Crouch = true;
	
	if(CD->getm_Keyboard()->GetDown(KEY_X))
		Crouch = false;

	//-- Actor's Aim
	if(CD->getm_Keyboard()->GetDown(KEY_E))
		Aim = true;
	
	if(CD->getm_Keyboard()->GetDown(KEY_R))
		Aim = false;
	
	if(CD->getm_Keyboard()->GetDown(VK_SPACE) && OnGround && !Crouch)
	{
		JumpHeight   = 5.0f;
		Acceleration = 0.0f;
		Jumping      = true;
	}

	if(Aim && CD->getm_Keyboard()->GetDown(VK_LBUTTON))
	{
		switch(CurrentMotion)
		{
			case RUN:
				Motion(RUNSHOOT);
				break;
			default:
				Motion(SHOOT);
				break;
		}
	}

	if(CD->getm_Keyboard()->GetDown(KEY_ESCAPE) && CD->getm_State() == STATE_MAINLOOP)
	{
		ShowCursor(true);
		CD->setState(STATE_MAINMENU);
	}
}

void Player::Update()
{
	if(!Jumping && GravityOn)
	{
		if(OnGround)
			StandGravity();
		else
			Gravity();
	}
	
	Animate();
	Motion(IDLE);
}

void Player::Animate()
{
	float start,end;

	geXForm3d_SetIdentity(&XForm);

	geXForm3d_RotateZ(&XForm,Rotation.Z);
	geXForm3d_RotateX(&XForm,0.0f);
	geXForm3d_RotateY(&XForm,Rotation.Y + 3.14159f);

	geXForm3d_Translate(&XForm,Translation.X,Translation.Y,Translation.Z);

	switch(CurrentMotion)
	{
		case AIM:
			if(Aim)
			{
				if(Crouch)
					motion = geActor_GetMotionByName(Def,"C_Aim");
				else
					motion = geActor_GetMotionByName(Def,"Aim");
			}
			break;
/*n		case C2IDLE:
			motion = geActor_GetMotionByName(def,"C2Idle");
			break;
		case FALL:
			motion = geActor_GetMotionByName(def,"Fall");
			break;
		case FALLSHOOT:
			motion = geActor_GetMotionByName(def,"FallShoot");
			break;
		case I2RUN:
			motion = geActor_GetMotionByName(def,"I2Run");
			break;
		case I2WALK:
			motion = geActor_GetMotionByName(def,"I2Walk");
			break;
*/		case IDLE:
			if(Aim && Crouch)
				motion = geActor_GetMotionByName(Def,"C_Aim");
			else
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"C_Idle");
			else
			if(Aim)
				motion = geActor_GetMotionByName(Def,"Aim");
			else
				motion = geActor_GetMotionByName(Def,"Idle");
			break;
		case JUMP:
			motion = geActor_GetMotionByName(Def,"Jump");
			break;
		case JUMPSHOOT:
			motion = geActor_GetMotionByName(Def,"JumpShoot");
			break;
		case JUMPSTART:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"C_JumpStart");
			else
				motion = geActor_GetMotionByName(Def,"JumpStart");
			break;
		case RUN:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"Crawl");
			else
				motion = geActor_GetMotionByName(Def,"Run");
		   	break;
		case RUNSHOOT:
			motion = geActor_GetMotionByName(Def,"RunShoot");
			break;
		case SHOOT:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"C_Shoot");
			else
				motion = geActor_GetMotionByName(Def,"Shoot");
			break;
		case STRAFELEFT:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"Crawl");
			else
				motion = geActor_GetMotionByName(Def,"StrafeLeft");
			break;
		case STRAFERIGHT:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"Crawl");
			else
				motion = geActor_GetMotionByName(Def,"StrafeLeft");
			break;
		case WALK:
			if(Crouch)
				motion = geActor_GetMotionByName(Def,"Crawl");
			else
				motion = geActor_GetMotionByName(Def,"Walk");
			break;
		case WALKSHOOT:
			motion = geActor_GetMotionByName(Def,"WalkShoot");
			break;
		default:
			break;
	}

	if(!CurrentMotion)
	{
		if(Crouch)
			motion = geActor_GetMotionByName(Def,"C_Idle");
		else
			motion = geActor_GetMotionByName(Def,"Idle");
	}

	geActor_SetPose(Actor,motion,Counter,&XForm);

	geMotion_GetTimeExtents(motion,&start,&end);

	Counter += 0.01f;

	if(Counter >= end)
	{
		switch(CurrentMotion)
		{
			case SHOOT:
				Motion(AIM);
				break;
			default:
				break;
		}

		Counter  = 0.0f;
	}
}

void Player::MoveToDeathMatchStart()
{
	geEntity           *entity;
	geEntity_EntitySet *entityset;
	DeathMatchStart    *start;

	entityset = geWorld_GetEntitySet(CD->getm_World()->getWorld(),"DeathMatchStart");

	if(entityset == NULL)
		return;
	
	entity = geEntity_EntitySetGetNextEntity(entityset,NULL);

	if(entity)
	{
		start = (DeathMatchStart *)geEntity_GetUserData(entity);

		Translation.X = start->Origin.X;
		Translation.Y = start->Origin.Y;
		Translation.Z = start->Origin.Z;
	}
}

void Player::MoveTo(float x,float y,float z)
{
	Translation.X = x;
	Translation.Y = y;
	Translation.Z = z;
}

void Player::Move(int reverse)
{
	geVec3d      newpos,savedpos,oldpos;
	geFloat      slide;
	geVec3d      saveddirection;
	GE_Collision collision;

	newpos = savedpos = oldpos = Translation;

	if(Crouch)
		geVec3d_AddScaled(&savedpos,&Direction,Speed * Reverse * 0.3f,&newpos);
	else
		geVec3d_AddScaled(&savedpos,&Direction,Speed * Reverse,&newpos);
	
	if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
	{
		if(collision.Actor)
		{
			ITEM_ENTITY_INFO *info = (ITEM_ENTITY_INFO *)geActor_GetUserData(collision.Actor);
			int type = EntityCollision(info);
			ProcessActorCollision(type,collision.Actor);
		}

		geXForm3d_SetIdentity(&XForm);

		geXForm3d_RotateY(&XForm,Rotation.Y);
		geXForm3d_GetUp(&XForm,&saveddirection);
		geVec3d_AddScaled(&oldpos,&saveddirection,StepHeight,&savedpos);
		geVec3d_AddScaled(&savedpos,&Direction,Speed * Reverse,&newpos);

		if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
		{
			if(collision.Plane.Normal.Y < 0)
			{
				slide = geVec3d_DotProduct(&newpos,&collision.Plane.Normal) + collision.Plane.Dist;

				newpos.X -= collision.Plane.Normal.X * slide;
				newpos.Y -= collision.Plane.Normal.Y * slide;
				newpos.Z -= collision.Plane.Normal.Z * slide;
			}
			else
			{
				slide = geVec3d_DotProduct(&newpos,&collision.Plane.Normal) - collision.Plane.Dist;

				newpos.X -= collision.Plane.Normal.X * slide;
				newpos.Y -= collision.Plane.Normal.Y * slide;
				newpos.Z -= collision.Plane.Normal.Z * slide;
			}

			if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
				newpos = collision.Impact;
		}
	}

	Translation = newpos;

	Gravity();
	Animate();
}

void Player::Move(int reverse,float speed)
{
	geVec3d      newpos,savedpos,oldpos;
	geFloat      slide;
	geVec3d      saveddirection;
	GE_Collision collision;

	newpos = savedpos = oldpos = Translation;

	geVec3d_AddScaled(&savedpos,&Direction,speed * Reverse,&newpos);

	if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
	{
		if(collision.Actor)
		{
			ITEM_ENTITY_INFO *info = (ITEM_ENTITY_INFO *)geActor_GetUserData(collision.Actor);
			int type = EntityCollision(info);
			ProcessActorCollision(type,collision.Actor);
		}
		
		geXForm3d_SetIdentity(&XForm);

		geXForm3d_SetYRotation(&XForm,Rotation.Y);
		geXForm3d_GetUp(&XForm,&saveddirection);
		geVec3d_AddScaled(&oldpos,&saveddirection,StepHeight,&savedpos);
		geVec3d_AddScaled(&savedpos,&Direction,Speed * Reverse * 0.7f,&newpos);

		if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
		{
			slide = geVec3d_DotProduct(&newpos,&collision.Plane.Normal) - collision.Plane.Dist;

			newpos.X -= collision.Plane.Normal.X * slide;
			newpos.Y -= collision.Plane.Normal.Y * slide;
			newpos.Z -= collision.Plane.Normal.Z * slide;

			if(geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&savedpos,&newpos,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision))
				newpos = collision.Impact;
		}
	}

	Translation = newpos;

	Gravity();
	Animate();
}

void Player::ProcessActorCollision(int type,geActor *actor)
{
	switch(type)
	{
		case ITEM_MEDKIT:
			CD->getm_Sound(SOUND_ACQUIRE)->Play();
			geWorld_RemoveActor(CD->getm_World()->getWorld(),actor);
			break;
		case ITEM_GRENADE:
			CD->getm_Sound(SOUND_ACQUIRE)->Play();
			geWorld_RemoveActor(CD->getm_World()->getWorld(),actor);
			break;
		case ITEM_ARMOR:
			CD->getm_Sound(SOUND_ACQUIRE)->Play();
			geWorld_RemoveActor(CD->getm_World()->getWorld(),actor);
			break;
		case ITEM_ROCKET:
			CD->getm_Sound(SOUND_ACQUIRE)->Play();
			geWorld_RemoveActor(CD->getm_World()->getWorld(),actor);
			break;
		case ITEM_SHREDDER:
			CD->getm_Sound(SOUND_ACQUIRE)->Play();
			geWorld_RemoveActor(CD->getm_World()->getWorld(),actor);
			break;
	//	case TYPE_DEMA:
	//		break;
		default:
			break;
	}
}

void Player::Forward()
{
	Reverse = 1;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetIn(&XForm,&Direction);

	Move(Reverse);
}

void Player::Backward()
{
	Reverse = -1;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetIn(&XForm,&Direction);

	Move(Reverse);
}

void Player::Left()
{
	Reverse = 1;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetLeft(&XForm,&Direction);

	Move(Reverse,0.5f);
}

void Player::Right()
{
	Reverse = -1;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetLeft(&XForm,&Direction);

	Move(Reverse,0.5f);
}

void Player::Jump()
{
	geVec3d      pos,ceiling;
	GE_Collision collision;

	Acceleration += 0.2f;

	pos = Translation;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetUp(&XForm,&Direction);

	geVec3d_AddScaled(&pos,&Direction,Acceleration,&ceiling);

	bool collide = geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&pos,&ceiling,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision);

	if(collide)
	{
		Jumping      = false;
		Acceleration = 0.0f;
	}
	else
	{
		Translation.Y += Acceleration;
	}

	if(Acceleration >= JumpHeight)
	{
		Acceleration = 0.0f;
		Jumping      = false;
		Falling      = true;
	}

	OnGround = false;
}

void Player::MouseLook()
{
	POINT   getpos,pos;
	geVec3d angle;
	float   turnspeed;
	float   updownspeed;

	GetCursorPos(&getpos);

	angle = Rotation;

	pos.x = CD->getm_Engine()->getWidth() / 2;
	pos.y = CD->getm_Engine()->getHeight() / 2;

	SetCursorPos(pos.x,pos.y);

	turnspeed   = abs(getpos.x - pos.x) * 0.0015f;
	updownspeed = abs(getpos.y - pos.y) * 0.0015f;
	
	if((getpos.x != pos.x) || (getpos.y != pos.y))
	{
		if(getpos.x > pos.x)
		{
			angle.Y = angle.Y - (geFloat)turnspeed;
			geXForm3d_RotateY(&XForm,angle.Y - Rotation.Y);
		}
		else
		if(getpos.x < pos.x)
		{
			angle.Y = angle.Y + (geFloat)turnspeed;
			geXForm3d_RotateY(&XForm,angle.Y + Rotation.Y);
		}

		if(getpos.y > pos.y)
			angle.X = angle.X + (geFloat)updownspeed;
		else
		if(getpos.y < pos.y)
			angle.X = angle.X - (geFloat)updownspeed;

		if(angle.X >  0.5f)
			angle.X = 0.5f;
		if(angle.X < -0.6f)
			angle.X = -0.6f;
	}

	Rotation = angle;
}

void Player::StandGravity()
{
	geVec3d      pos;
	geVec3d      floor;
	GE_Collision collision;

	pos = Translation;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetUp(&XForm,&Direction);

	geVec3d_AddScaled(&pos,&Direction,-1.0f,&floor);

	bool collide = geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&pos,&floor,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision);

	if(collide)
	{
		floor        = collision.Impact;
		OnGround     = true;
		Acceleration = 0.0f;
	}
	else
		OnGround = false;

	floor.Y    += 1;
	Translation = floor;
}

void Player::Gravity()
{
	geVec3d      pos;
	geVec3d      floor;
	GE_Collision collision;

	if(Falling)
		Acceleration = 2.0f;
	else
		Acceleration = 4.0f;

	pos = Translation;

	geXForm3d_SetIdentity(&XForm);
	geXForm3d_SetYRotation(&XForm,Rotation.Y);
	geXForm3d_GetUp(&XForm,&Direction);

	geVec3d_AddScaled(&pos,&Direction,-Acceleration,&floor);

	bool collide = geWorld_Collision(CD->getm_World()->getWorld(),&Box.Min,&Box.Max,&pos,&floor,GE_CONTENTS_SOLID_CLIP,GE_COLLIDE_ALL,0xffffffff,NULL,NULL,&collision);

	if(collide)
	{
		floor        = collision.Impact;
		OnGround     = true;
		Jumping      = false;
		Falling      = false;
		Acceleration = 0.0f;
	}

	floor.Y    += 1.0f;
	Translation = floor;
}

void Player::Motion(int motion)
{
	CurrentMotion = motion;
}